% M-file 9.8
% CepstrumCrossSynthesis.m
%
% cross-synthesis with cepstrum

clear all; close all

%----- USER DATA -----
%[DAFx_in1, SR] = wavread('didge_court.wav');   % sound 1: excitation
%DAFx_in2       = wavread('la.wav'); % sound 2: spectral shape
[DAFx_in1, SR] = wavread('moore_guitar.wav'); % sound 1: excitation
DAFx_in2 = wavread('toms_diner.wav'); % sound 2: spectral env.
WindowLength   = 1024;            % window size
n1             = 256;             % hop size
ordre1         = 30;              % cut quefrency for sound 1
ordre2         = 30;              % cut quefrency for sound 2


%----- initialisations -----
w1             = hanningz(WindowLength);  % analysis window
w2             = w1;              % synthesis window
WindowLength2  = WindowLength/2
grain1         = zeros(WindowLength,1);
grain2         = zeros(WindowLength,1);
pin            = 0;               % start index
L              = min(length(DAFx_in1),length(DAFx_in2));
pend           = L - WindowLength; % end index
DAFx_in1       = [zeros(WindowLength, 1); DAFx_in1; ...
   zeros(WindowLength-mod(L,n1),1)] / max(abs(DAFx_in1));
DAFx_in2       = [zeros(WindowLength, 1); DAFx_in2; ...
   zeros(WindowLength-mod(L,n1),1)] / max(abs(DAFx_in2));
DAFx_out       = zeros(L,1);

%----- cross synthesis -----
while pin<pend
   grain1     = DAFx_in1(pin+1:pin+WindowLength).* w1;
   grain2     = DAFx_in2(pin+1:pin+WindowLength).* w1;
%===========================================
   f1         = fft(grain1);

   f          = fft(grain2)/WindowLength2;
   flog       = log(0.00001+abs(f));
   cep        = ifft(flog);                 % cepstrum of sound 2
   cep_coupe  = [cep(1)/2; cep(2:ordre1); zeros(WindowLength-ordre1,1)];
   flog_coupe = 2*real(fft(cep_coupe));
   f2         = exp(flog_coupe);            % spectral shape of sound 2

   grain      = (real(ifft(f1.*f2))).*w2;   % resynthesis grain
% ===========================================
   DAFx_out(pin+1:pin+WindowLength) = DAFx_out(pin+1:pin+WindowLength) + grain;
   pin        = pin + n1;
end

%----- listening and saving the output -----
%DAFx_in = DAFx_in(WindowLength+1:WindowLength+L);
DAFx_out = DAFx_out(WindowLength+1:length(DAFx_out)) / max(abs(DAFx_out));
soundsc(DAFx_out, SR);
DAFx_out_norm = .99* DAFx_out/max(abs(DAFx_out)); % scale for wav output
wavwrite(DAFx_out_norm, SR, 'CrossCepstrum')
