% M-file 9.13
% UX_pitch_pv_move.m

clear; 

%----- USER DATA -----
[DAFx_in, SR] = wavread('la.wav'); % sound file
n1            = 512;  % analysis hop size, try n1=400 (pitch down) or 150 (pitch up)
n2            = 256;  % synthesis hop size, keep it a divisor of WindowLength (256 is pretty good)
WindowLength  = 2048; % window length
w1            = hanningz(WindowLength); % analysis window
w2            = w1;   % synthesis window
order         = 50;   % cut quefrency

%----- initialisations -----
ral           = n2/n1;
WindowLength2 = WindowLength/2;
% for linear interpolation of a grain of length WindowLength
lx            = floor(WindowLength*n1/n2);
x             = 1 + (0:lx-1)'*WindowLength/lx;
ix            = floor(x);
ix1           = ix + 1;
dx            = x - ix;
dx1           = 1 - dx;
warping_coef  = n1/n2
t             = 1 + floor((0:WindowLength-1)*warping_coef);
lmax          = max(WindowLength,t(WindowLength))
L             = length(DAFx_in);
DAFx_in       = [zeros(WindowLength, 1); DAFx_in; ...
   zeros(WindowLength-mod(L,n1),1)] / max(abs(DAFx_in));
DAFx_out      = zeros(lx+length(DAFx_in),1);
omega         = 2*pi*n1*[0:WindowLength-1]'/WindowLength;
phi0          = zeros(WindowLength,1);
psi           = zeros(WindowLength,1);

tic
%UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
pin    = 0;
pout   = 0;
pend   = L - lmax;

while pin<pend
   grain  = DAFx_in(pin+1:pin+WindowLength).* w1;
%===========================================
   f      = fft(fftshift(grain));
   r      = abs(f);
   phi    = angle(f);
   
   delta_phi = omega + princarg(phi-phi0-omega); % unwrapping the phase
   phi0   = phi;
   psi    = princarg(psi+delta_phi*ral);

   %----- formant move -----
   grain1 = DAFx_in(pin+t) .* w1;
   f1     = fft(grain1)/WindowLength2;
   flog   = log(0.00001+abs(f1))-log(0.00001+abs(f));
   cep    = ifft(flog);
   cep_coupe = [cep(1)/2; cep(2:order); zeros(WindowLength-order,1)];
   corr   = exp(2*real(fft(cep_coupe)));

   ft     = (r.* corr.* exp(i*psi));
   grain  = fftshift(real(ifft(ft))).*w2;

   %----- interpolation -----
   grain2 = [grain;0];
   grain3 = grain2(ix).*dx1+grain2(ix1).*dx;
   % plot(grain);drawnow;
%===========================================
   DAFx_out(pout+1:pout+lx) = DAFx_out(pout+1:pout+lx) + grain3;
   pin    = pin + n1;
   pout   = pout + n1;
end
%UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
toc

%----- listening and saving the output -----
DAFx_in = DAFx_in(WindowLength+1:WindowLength+L);
DAFx_out = DAFx_out(WindowLength+1:WindowLength+L) / max(abs(DAFx_out));
soundsc(DAFx_out, SR);
wavwrite(DAFx_out, SR, 'la_pitch_pv_move.wav');
