function [rxx_norm, rxx, rxx0]=xcorr_norm(xp, lmin, lmax, Nblock)
% [rxx_norm, rxx, rxx0]=xcorr_norm(xp, lmin, lmax, Nblock)
%
% M-file 9.22
% xcorr_norm.m
%
% Computation of exact autocorrelation and normalized autocorrelation.
% Used for pitch estimation based on long-term prediction (LTP).
% xp:       input block of length Nb+lmax, including lmax pre-samples
% lmin:     min. tested pitch lag
% lmax:     max. tested pitch lag
% Nblock:   block length
% rxx_norm: rxx_all normalized on energy of delayed block (for long-term-prediction)
% rxx:      autocorrelation with use of pre-samples, use of exact delayed signal
% rxx_0:    energy of delayed blocks
%
% all return values are vectors for the lag range lmin:lmax
% calc. normalized autocorrelation
% 
% (c) 2002 Florian Keiler

x        = xp((1:Nblock)+lmax); % input block without pre-samples
lags     = lmin:lmax;           % tested lag range
Nlag     = length(lags);        % no. of lags
rxx      = zeros(1,Nlag);       % autocorr. sequence
rxx0     = zeros(1,Nlag);       % energy of delayed blocks
rxx_norm = zeros(1,Nlag);       % normalized autocorr. sequence

for l=1:Nlag
    ii=lags(l); %tested lag
    rxx0(l)=sum(xp((1:Nblock)+lmax-lags(l)).^2); % energy of delayed block
    rxx(l)=sum(x.*xp((1:Nblock)+lmax-lags(l)));  % exact autocorrelation
end
rxx_norm=rxx.^2./rxx0; % normalized autocorrelation, to be maximized
 
